# -*- coding: utf-8 -*-
# Generated by Django 1.9.8 on 2016-07-22 19:53
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_celery_fulldbresult', '0004_auto_20150914_1947'),
    ]

    operations = [
        migrations.AddField(
            model_name='taskresultmeta',
            name='scheduled_id',
            field=models.UUIDField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='taskresultmeta',
            name='status',
            field=models.CharField(choices=[('FAILURE', 'FAILURE'), ('PENDING', 'PENDING'), ('RECEIVED', 'RECEIVED'), ('RETRY', 'RETRY'), ('REVOKED', 'REVOKED'), ('SCHEDULED', 'SCHEDULED'), ('SCHEDULED_SENT', 'SCHEDULED_SENT'), ('STARTED', 'STARTED'), ('SUCCESS', 'SUCCESS')], default='PENDING', max_length=50, verbose_name='state'),
        ),
    ]
